from oarepo_runtime.services.search import I18nRDMSearchOptions

from . import facets


class DataSearchOptions(I18nRDMSearchOptions):
    """DataRecord search options."""

    facet_groups = {}

    facets = {
        "access_embargo_active": facets.access_embargo_active,
        "access_embargo_until": facets.access_embargo_until,
        "access_files": facets.access_files,
        "access_record": facets.access_record,
        "access_status": facets.access_status,
        "metadata_abstract_cs": facets.metadata_abstract_cs,
        "metadata_abstract_en": facets.metadata_abstract_en,
        "metadata_abstract_lang": facets.metadata_abstract_lang,
        "metadata_accessibility_cs": facets.metadata_accessibility_cs,
        "metadata_accessibility_en": facets.metadata_accessibility_en,
        "metadata_accessibility_lang": facets.metadata_accessibility_lang,
        "metadata_additionalTitles_title_cs": facets.metadata_additionalTitles_title_cs,
        "metadata_additionalTitles_title_en": facets.metadata_additionalTitles_title_en,
        "metadata_additionalTitles_title_lang": (
            facets.metadata_additionalTitles_title_lang
        ),
        "metadata_additionalTitles_titleType": (
            facets.metadata_additionalTitles_titleType
        ),
        "metadata_contributors_affiliations": facets.metadata_contributors_affiliations,
        "metadata_contributors_person_or_org_family_name": (
            facets.metadata_contributors_person_or_org_family_name
        ),
        "metadata_contributors_person_or_org_given_name": (
            facets.metadata_contributors_person_or_org_given_name
        ),
        "metadata_contributors_person_or_org_identifiers_identifier": (
            facets.metadata_contributors_person_or_org_identifiers_identifier
        ),
        "metadata_contributors_person_or_org_identifiers_scheme": (
            facets.metadata_contributors_person_or_org_identifiers_scheme
        ),
        "metadata_contributors_person_or_org_name": (
            facets.metadata_contributors_person_or_org_name
        ),
        "metadata_contributors_person_or_org_type": (
            facets.metadata_contributors_person_or_org_type
        ),
        "metadata_contributors_role": facets.metadata_contributors_role,
        "metadata_creators_affiliations": facets.metadata_creators_affiliations,
        "metadata_creators_person_or_org_family_name": (
            facets.metadata_creators_person_or_org_family_name
        ),
        "metadata_creators_person_or_org_given_name": (
            facets.metadata_creators_person_or_org_given_name
        ),
        "metadata_creators_person_or_org_identifiers_identifier": (
            facets.metadata_creators_person_or_org_identifiers_identifier
        ),
        "metadata_creators_person_or_org_identifiers_scheme": (
            facets.metadata_creators_person_or_org_identifiers_scheme
        ),
        "metadata_creators_person_or_org_name": (
            facets.metadata_creators_person_or_org_name
        ),
        "metadata_creators_person_or_org_type": (
            facets.metadata_creators_person_or_org_type
        ),
        "metadata_creators_role": facets.metadata_creators_role,
        "metadata_dateAvailable": facets.metadata_dateAvailable,
        "metadata_dateIssued": facets.metadata_dateIssued,
        "metadata_dateValidTo": facets.metadata_dateValidTo,
        "metadata_dateWithdrawn_dateInformation": (
            facets.metadata_dateWithdrawn_dateInformation
        ),
        "metadata_dateWithdrawn_type": facets.metadata_dateWithdrawn_type,
        "metadata_events_eventLocation_country": (
            facets.metadata_events_eventLocation_country
        ),
        "metadata_events_eventLocation_place": (
            facets.metadata_events_eventLocation_place
        ),
        "metadata_funders_award": facets.metadata_funders_award,
        "metadata_funders_funder": facets.metadata_funders_funder,
        "metadata_geoLocations_geoLocationPlace": (
            facets.metadata_geoLocations_geoLocationPlace
        ),
        "metadata_geoLocations_geoLocationPoint_pointLatitude": (
            facets.metadata_geoLocations_geoLocationPoint_pointLatitude
        ),
        "metadata_geoLocations_geoLocationPoint_pointLongitude": (
            facets.metadata_geoLocations_geoLocationPoint_pointLongitude
        ),
        "metadata_languages": facets.metadata_languages,
        "metadata_methods_cs": facets.metadata_methods_cs,
        "metadata_methods_en": facets.metadata_methods_en,
        "metadata_methods_lang": facets.metadata_methods_lang,
        "metadata_objectIdentifiers_identifier": (
            facets.metadata_objectIdentifiers_identifier
        ),
        "metadata_objectIdentifiers_scheme": facets.metadata_objectIdentifiers_scheme,
        "metadata_originalRecord": facets.metadata_originalRecord,
        "metadata_publishers": facets.metadata_publishers,
        "metadata_relatedItems_itemContributors_affiliations": (
            facets.metadata_relatedItems_itemContributors_affiliations
        ),
        "metadata_relatedItems_itemContributors_person_or_org_family_name": (
            facets.metadata_relatedItems_itemContributors_person_or_org_family_name
        ),
        "metadata_relatedItems_itemContributors_person_or_org_given_name": (
            facets.metadata_relatedItems_itemContributors_person_or_org_given_name
        ),
        "metadata_relatedItems_itemContributors_person_or_org_identifiers_identifier": (
            facets.metadata_relatedItems_itemContributors_person_or_org_identifiers_identifier
        ),
        "metadata_relatedItems_itemContributors_person_or_org_identifiers_scheme": (
            facets.metadata_relatedItems_itemContributors_person_or_org_identifiers_scheme
        ),
        "metadata_relatedItems_itemContributors_person_or_org_name": (
            facets.metadata_relatedItems_itemContributors_person_or_org_name
        ),
        "metadata_relatedItems_itemContributors_person_or_org_type": (
            facets.metadata_relatedItems_itemContributors_person_or_org_type
        ),
        "metadata_relatedItems_itemContributors_role": (
            facets.metadata_relatedItems_itemContributors_role
        ),
        "metadata_relatedItems_itemCreators_affiliations": (
            facets.metadata_relatedItems_itemCreators_affiliations
        ),
        "metadata_relatedItems_itemCreators_person_or_org_family_name": (
            facets.metadata_relatedItems_itemCreators_person_or_org_family_name
        ),
        "metadata_relatedItems_itemCreators_person_or_org_given_name": (
            facets.metadata_relatedItems_itemCreators_person_or_org_given_name
        ),
        "metadata_relatedItems_itemCreators_person_or_org_identifiers_identifier": (
            facets.metadata_relatedItems_itemCreators_person_or_org_identifiers_identifier
        ),
        "metadata_relatedItems_itemCreators_person_or_org_identifiers_scheme": (
            facets.metadata_relatedItems_itemCreators_person_or_org_identifiers_scheme
        ),
        "metadata_relatedItems_itemCreators_person_or_org_name": (
            facets.metadata_relatedItems_itemCreators_person_or_org_name
        ),
        "metadata_relatedItems_itemCreators_person_or_org_type": (
            facets.metadata_relatedItems_itemCreators_person_or_org_type
        ),
        "metadata_relatedItems_itemCreators_role": (
            facets.metadata_relatedItems_itemCreators_role
        ),
        "metadata_relatedItems_itemEndPage": facets.metadata_relatedItems_itemEndPage,
        "metadata_relatedItems_itemIssue": facets.metadata_relatedItems_itemIssue,
        "metadata_relatedItems_itemPIDs_identifier": (
            facets.metadata_relatedItems_itemPIDs_identifier
        ),
        "metadata_relatedItems_itemPIDs_scheme": (
            facets.metadata_relatedItems_itemPIDs_scheme
        ),
        "metadata_relatedItems_itemPublisher": (
            facets.metadata_relatedItems_itemPublisher
        ),
        "metadata_relatedItems_itemRelationType": (
            facets.metadata_relatedItems_itemRelationType
        ),
        "metadata_relatedItems_itemResourceType": (
            facets.metadata_relatedItems_itemResourceType
        ),
        "metadata_relatedItems_itemStartPage": (
            facets.metadata_relatedItems_itemStartPage
        ),
        "metadata_relatedItems_itemURL": facets.metadata_relatedItems_itemURL,
        "metadata_relatedItems_itemVolume": facets.metadata_relatedItems_itemVolume,
        "metadata_relatedItems_itemYear": facets.metadata_relatedItems_itemYear,
        "metadata_resourceType": facets.metadata_resourceType,
        "metadata_rights": facets.metadata_rights,
        "metadata_series_seriesTitle": facets.metadata_series_seriesTitle,
        "metadata_series_seriesVolume": facets.metadata_series_seriesVolume,
        "metadata_subjectCategories": facets.metadata_subjectCategories,
        "metadata_subjects_classificationCode": (
            facets.metadata_subjects_classificationCode
        ),
        "metadata_subjects_subject_cs": facets.metadata_subjects_subject_cs,
        "metadata_subjects_subject_en": facets.metadata_subjects_subject_en,
        "metadata_subjects_subject_lang": facets.metadata_subjects_subject_lang,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_valueURI": facets.metadata_subjects_valueURI,
        "metadata_systemIdentifiers_identifier": (
            facets.metadata_systemIdentifiers_identifier
        ),
        "metadata_systemIdentifiers_scheme": facets.metadata_systemIdentifiers_scheme,
        "metadata_technicalInfo_cs": facets.metadata_technicalInfo_cs,
        "metadata_technicalInfo_en": facets.metadata_technicalInfo_en,
        "metadata_technicalInfo_lang": facets.metadata_technicalInfo_lang,
        "metadata_version": facets.metadata_version,
        **getattr(I18nRDMSearchOptions, "facets", {}),
        "record_status": facets.record_status,
        "has_draft": facets.has_draft,
    }



import marshmallow as ma
from marshmallow import Schema
from marshmallow import fields as ma_fields
from marshmallow.validate import OneOf
from oarepo_runtime.services.schema.i18n_ui import (
    I18nStrUIField,
    MultilingualLocalizedUIField,
    MultilingualUIField,
)
from oarepo_runtime.services.schema.marshmallow import DictOnlySchema
from oarepo_runtime.services.schema.rdm_ui import (
    RDMCreatorsUISchema,
    RDMFundersUISchema,
)
from oarepo_runtime.services.schema.ui import (
    InvenioRDMUISchema,
    LocalizedDate,
    LocalizedEDTF,
)

from nr_metadata.common.services.records.ui_schema_datatypes import (
    NREventUISchema,
    NRGeoLocationUISchema,
    NRLanguageVocabularyUISchema,
    NRRelatedItemUISchema,
    NRResourceTypeVocabularyUISchema,
    NRRightsVocabularyUISchema,
    NRSeriesUISchema,
    NRSubjectCategoryVocabularyUISchema,
    NRSubjectUISchema,
)
from nr_metadata.ui_schema.identifiers import (
    NRObjectIdentifierUISchema,
    NRSystemIdentifierUISchema,
)
from nr_metadata.ui_schema.subjects import NRSubjectListField


class NRCommonRecordUISchema(InvenioRDMUISchema):
    class Meta:
        unknown = ma.RAISE

    deletion_status = ma_fields.String()

    is_deleted = ma_fields.Boolean()

    is_published = ma_fields.Boolean()

    metadata = ma_fields.Nested(lambda: NRCommonMetadataUISchema())

    version_id = ma_fields.Integer()


class NRCommonMetadataUISchema(Schema):
    class Meta:
        unknown = ma.RAISE

    abstract = MultilingualUIField(I18nStrUIField())

    accessibility = MultilingualLocalizedUIField(I18nStrUIField())

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesUISchema())
    )

    contributors = ma_fields.List(ma_fields.Nested(lambda: RDMCreatorsUISchema()))

    creators = ma_fields.List(
        ma_fields.Nested(lambda: RDMCreatorsUISchema()), required=True
    )

    dateAvailable = LocalizedDate()

    dateIssued = LocalizedEDTF()

    events = ma_fields.List(ma_fields.Nested(lambda: NREventUISchema()))

    funders = ma_fields.List(ma_fields.Nested(lambda: RDMFundersUISchema()))

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationUISchema()))

    languages = ma_fields.List(ma_fields.Nested(lambda: NRLanguageVocabularyUISchema()))

    methods = MultilingualUIField(I18nStrUIField())

    notes = ma_fields.List(ma_fields.String())

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRObjectIdentifierUISchema())
    )

    originalRecord = ma_fields.String()

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemUISchema()))

    resourceType = ma_fields.Nested(
        lambda: NRResourceTypeVocabularyUISchema(), required=True
    )

    rights = ma_fields.Nested(lambda: NRRightsVocabularyUISchema())

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesUISchema()))

    subjectCategories = ma_fields.List(
        ma_fields.Nested(lambda: NRSubjectCategoryVocabularyUISchema())
    )

    subjects = NRSubjectListField(ma_fields.Nested(lambda: NRSubjectUISchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierUISchema())
    )

    technicalInfo = MultilingualUIField(I18nStrUIField())

    title = ma_fields.String(required=True)

    version = ma_fields.String()


class AdditionalTitlesUISchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    title = I18nStrUIField(required=True)

    titleType = ma_fields.String(
        required=True,
        validate=[OneOf(["translatedTitle", "alternativeTitle", "subtitle", "other"])],
    )



from invenio_drafts_resources.services.records.schema import (
    ParentSchema as InvenioParentSchema,
)


class GeneratedParentSchema(InvenioParentSchema):
    """"""

    owners = ma.fields.List(ma.fields.Dict(), load_only=True)


class NRCommonRecordSchema(RDMBaseRecordSchema):
    parent = ma.fields.Nested(GeneratedParentSchema)



import marshmallow as ma
from marshmallow import Schema
from marshmallow import fields as ma_fields
from oarepo_runtime.services.schema.ui import InvenioRDMUISchema

from nr_metadata.datacite.services.records.ui_schema_datatypes import (
    AlternateIdentifierUISchema,
    ContainerUISchema,
    ContributorUISchema,
    CreatorUISchema,
    DateUISchema,
    DescriptionUISchema,
    FundingReferenceUISchema,
    GeoLocationUISchema,
    PublisherUISchema,
    RelatedIdentifierUISchema,
    RelatedItemUISchema,
    ResourceTypeUISchema,
    RightsUISchema,
    SubjectUISchema,
    TitleUISchema,
)


class DataCiteRecordUISchema(InvenioRDMUISchema):
    class Meta:
        unknown = ma.RAISE

    deletion_status = ma_fields.String()

    is_deleted = ma_fields.Boolean()

    is_published = ma_fields.Boolean()

    metadata = ma_fields.Nested(lambda: NRDataCiteMetadataUISchema())

    version_id = ma_fields.Integer()


class NRDataCiteMetadataUISchema(Schema):
    class Meta:
        unknown = ma.RAISE

    alternateIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: AlternateIdentifierUISchema())
    )

    container = ma_fields.Nested(lambda: ContainerUISchema())

    contributors = ma_fields.List(ma_fields.Nested(lambda: ContributorUISchema()))

    creators = ma_fields.List(ma_fields.Nested(lambda: CreatorUISchema()))

    dates = ma_fields.List(ma_fields.Nested(lambda: DateUISchema()))

    descriptions = ma_fields.List(ma_fields.Nested(lambda: DescriptionUISchema()))

    doi = ma_fields.String()

    formats = ma_fields.List(ma_fields.String())

    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: FundingReferenceUISchema())
    )

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: GeoLocationUISchema()))

    language = ma_fields.String()

    publicationYear = ma_fields.String()

    publisher = ma_fields.Nested(lambda: PublisherUISchema())

    relatedIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: RelatedIdentifierUISchema())
    )

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: RelatedItemUISchema()))

    resourceType = ma_fields.Nested(lambda: ResourceTypeUISchema())

    rightsList = ma_fields.List(ma_fields.Nested(lambda: RightsUISchema()))

    schemaVersion = ma_fields.String()

    sizes = ma_fields.List(ma_fields.String())

    subjects = ma_fields.List(ma_fields.Nested(lambda: SubjectUISchema()))

    titles = ma_fields.List(ma_fields.Nested(lambda: TitleUISchema()))

    url = ma_fields.String()

    version = ma_fields.String()



from invenio_records.dumpers import SearchDumperExt


class SyntheticFieldsDumperExtension(SearchDumperExt):
    paths = []

    def dump(self, record, data):
        """Dump the data."""
        data["syntheticFields"] = {}

        metadata = data.get("metadata", {})

        # Person synthetic field.
        person = []
        for item in metadata.get("contributors", []):
            person.append(item["fullName"])

        for item in metadata.get("creators", []):
            person.append(item["fullName"])

        data["syntheticFields"]["person"] = person

        # Institutions synthetic field.
        institutions = []
        for item in metadata.get("creators", []):
            if "affiliations" in item:
                institutions.extend(item["affiliations"])

        for item in metadata.get("contributors", []):
            if "affiliations" in item:
                institutions.extend(item["affiliations"])

        institutions.extend(metadata.get("thesis", {}).get("degreeGrantors", []))

        data["syntheticFields"]["institutions"] = institutions

        # Keywords synthetic fields. Note: called subjects in metadata.
        (
            data["syntheticFields"]["keywords_cs"],
            data["syntheticFields"]["keywords_en"],
        ) = ([], [])
        all_subjects = [
            item
            for subject in metadata.get("subjects", [])
            for item in subject.get("subject", [])
        ]
        for subject in all_subjects:
            if "lang" not in subject or "value" not in subject["lang"]:
                continue
            
            if subject["lang"] == "cs":
                data["syntheticFields"]["keywords_cs"].append(subject["value"])

            if subject["lang"] == "en":
                data["syntheticFields"]["keywords_en"].append(subject["value"])

    def load(self, data, record_cls):
        """Load the data.

        Reverse the changes made by the dump method.
        """

        data.pop("syntheticFields", None)



import marshmallow as ma
from edtf import Interval as EDTFInterval
from invenio_rdm_records.services.schemas.metadata import CreatorSchema
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import fields as ma_fields
from marshmallow.fields import String
from marshmallow_utils.fields import TrimmedString
from oarepo_runtime.services.schema.i18n import I18nStrField, MultilingualField
from oarepo_runtime.services.schema.marshmallow import DictOnlySchema
from oarepo_runtime.services.schema.validation import (
    CachedMultilayerEDTFValidator,
    validate_identifier,
)

from nr_metadata.schema.identifiers import NRObjectIdentifierSchema


class NREventSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    eventDate = TrimmedString(
        required=True, validate=[CachedMultilayerEDTFValidator(types=(EDTFInterval,))]
    )

    eventLocation = ma_fields.Nested(lambda: NRLocationSchema(), required=True)

    eventNameAlternate = ma_fields.List(ma_fields.String())

    eventNameOriginal = ma_fields.String(required=True)


class NRGeoLocationSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    geoLocationPlace = ma_fields.String()

    geoLocationPoint = ma_fields.Nested(lambda: NRGeoLocationPointSchema())


class NRLocationSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    country = ma_fields.Nested(lambda: NRCountryVocabularySchema())

    place = ma_fields.String(required=True)


class NRRelatedItemSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    itemContributors = ma_fields.List(ma_fields.Nested(lambda: CreatorSchema()))

    itemCreators = ma_fields.List(ma_fields.Nested(lambda: CreatorSchema()))

    itemEndPage = ma_fields.String()

    itemIssue = ma_fields.String()

    itemPIDs = ma_fields.List(
        ma_fields.Nested(
            lambda: NRObjectIdentifierSchema(),
            validate=[lambda value: validate_identifier(value)],
        )
    )

    itemPublisher = ma_fields.String()

    itemRelationType = ma_fields.Nested(lambda: NRItemRelationTypeVocabularySchema())

    itemResourceType = ma_fields.Nested(lambda: NRResourceTypeVocabularySchema())

    itemStartPage = ma_fields.String()

    itemTitle = ma_fields.String(required=True)

    itemURL = ma_fields.String()

    itemVolume = ma_fields.String()

    itemYear = ma_fields.Integer()


class NRCountryVocabularySchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRGeoLocationPointSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    pointLatitude = ma_fields.Float(
        required=True, validate=[ma.validate.Range(min=-90.0, max=90.0)]
    )

    pointLongitude = ma_fields.Float(
        required=True, validate=[ma.validate.Range(min=-180.0, max=180.0)]
    )


class NRItemRelationTypeVocabularySchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRLanguageVocabularySchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRResourceTypeVocabularySchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRRightsVocabularySchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRSeriesSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    seriesTitle = ma_fields.String(required=True)

    seriesVolume = ma_fields.String()


class NRSubjectCategoryVocabularySchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    title = i18n_strings


class NRSubjectSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    classificationCode = ma_fields.String()

    subject = MultilingualField(I18nStrField(), required=True)

    subjectScheme = ma_fields.String()

    valueURI = ma_fields.String()


class NRExternalLocationSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    externalLocationNote = ma_fields.String()

    externalLocationURL = ma_fields.String(required=True)



import marshmallow as ma
from invenio_drafts_resources.services.records.schema import (
    ParentSchema as InvenioParentSchema,
)
from invenio_rdm_records.services.schemas.access import AccessSchema
from invenio_rdm_records.services.schemas.pids import PIDSchema
from invenio_rdm_records.services.schemas.record import validate_scheme
from marshmallow import Schema
from marshmallow import fields as ma_fields
from marshmallow.fields import Dict, Nested
from marshmallow_utils.fields import SanitizedUnicode
from marshmallow_utils.fields.nestedattr import NestedAttribute
from oarepo_runtime.services.schema.marshmallow import RDMBaseRecordSchema

from nr_metadata.datacite.services.records.schema_datatypes import (
    AlternateIdentifierSchema,
    ContainerSchema,
    ContributorSchema,
    CreatorSchema,
    DateSchema,
    DescriptionSchema,
    FundingReferenceSchema,
    GeoLocationSchema,
    PublisherSchema,
    RelatedIdentifierSchema,
    RelatedItemSchema,
    ResourceTypeSchema,
    RightsSchema,
    SubjectSchema,
    TitleSchema,
)


class GeneratedParentSchema(InvenioParentSchema):
    """"""

    owners = ma.fields.List(ma.fields.Dict(), load_only=True)


class DataCiteRecordSchema(RDMBaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    access = NestedAttribute(lambda: AccessSchema())

    metadata = ma_fields.Nested(lambda: NRDataCiteMetadataSchema())

    pids = Dict(
        keys=SanitizedUnicode(validate=validate_scheme),
        values=Nested(PIDSchema),
    )
    parent = ma.fields.Nested(GeneratedParentSchema)


class NRDataCiteMetadataSchema(Schema):
    class Meta:
        unknown = ma.RAISE

    alternateIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: AlternateIdentifierSchema())
    )

    container = ma_fields.Nested(lambda: ContainerSchema())

    contributors = ma_fields.List(ma_fields.Nested(lambda: ContributorSchema()))

    creators = ma_fields.List(ma_fields.Nested(lambda: CreatorSchema()))

    dates = ma_fields.List(ma_fields.Nested(lambda: DateSchema()))

    descriptions = ma_fields.List(ma_fields.Nested(lambda: DescriptionSchema()))

    doi = ma_fields.String()

    formats = ma_fields.List(ma_fields.String())

    fundingReferences = ma_fields.List(
        ma_fields.Nested(lambda: FundingReferenceSchema())
    )

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: GeoLocationSchema()))

    language = ma_fields.String()

    publicationYear = ma_fields.String()

    publisher = ma_fields.Nested(lambda: PublisherSchema())

    relatedIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: RelatedIdentifierSchema())
    )

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: RelatedItemSchema()))

    resourceType = ma_fields.Nested(lambda: ResourceTypeSchema())

    rightsList = ma_fields.List(ma_fields.Nested(lambda: RightsSchema()))

    schemaVersion = ma_fields.String()

    sizes = ma_fields.List(ma_fields.String())

    subjects = ma_fields.List(ma_fields.Nested(lambda: SubjectSchema()))

    titles = ma_fields.List(ma_fields.Nested(lambda: TitleSchema()))

    url = ma_fields.String()

    version = ma_fields.String()



from invenio_assets.webpack import WebpackThemeBundle

theme = WebpackThemeBundle(
    __name__,
    ".",
    default="semantic-ui",
    themes={
        "semantic-ui": {
            "entry": {"nr_components": "./js/nr/custom-components.js"},
            "dependencies": {},
            "devDependencies": {},
            "aliases": {
                "@nr": "./js/nr",
                "@translations/nr/i18next": "translations/nr_metadata/i18next.js",
            },
        }
    },
)



from invenio_drafts_resources.records.api import DraftRecordIdProviderV2
from invenio_drafts_resources.services.records.components.media_files import (
    MediaFilesAttrConfig,
)
from invenio_rdm_records.records.api import RDMMediaFileRecord, RDMParent, RDMRecord
from invenio_records.systemfields import ConstantField
from invenio_records_resources.records.systemfields import FilesField, IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext

from nr_metadata.datacite.records.dumpers.dumper import DataciteDumper
from nr_metadata.datacite.records.models import DataciteMetadata, DataciteParentMetadata


class DataciteParentRecord(RDMParent):
    model_cls = DataciteParentMetadata


class DataciteIdProvider(DraftRecordIdProviderV2):
    pid_type = "dtct"


class DataciteRecord(RDMRecord):

    model_cls = DataciteMetadata

    schema = ConstantField("$schema", "local://datacite-1.0.0.json")

    index = IndexField(
        "datacite-datacite-1.0.0",
    )

    pid = PIDField(
        provider=DataciteIdProvider, context_cls=PIDFieldContext, create=True
    )

    dumper = DataciteDumper()

    media_files = FilesField(
        key=MediaFilesAttrConfig["_files_attr_key"],
        bucket_id_attr=MediaFilesAttrConfig["_files_bucket_id_attr_key"],
        bucket_attr=MediaFilesAttrConfig["_files_bucket_attr_key"],
        store=False,
        dump=False,
        file_cls=RDMMediaFileRecord,
        create=False,
        delete=False,
    )


class RDMRecordMediaFiles(DataciteRecord):
    """RDM Media file record API."""

    files = FilesField(
        key=MediaFilesAttrConfig["_files_attr_key"],
        bucket_id_attr=MediaFilesAttrConfig["_files_bucket_id_attr_key"],
        bucket_attr=MediaFilesAttrConfig["_files_bucket_attr_key"],
        store=False,
        dump=False,
        file_cls=RDMMediaFileRecord,
        # Don't create
        create=False,
        # Don't delete, we'll manage in the service
        delete=False,
    )


RDMMediaFileRecord.record_cls = RDMRecordMediaFiles



from oarepo_runtime.services.search import I18nRDMSearchOptions

from . import facets


class DataciteSearchOptions(I18nRDMSearchOptions):
    """DataciteRecord search options."""

    facet_groups = {}

    facets = {
        "access_embargo_active": facets.access_embargo_active,
        "access_embargo_until": facets.access_embargo_until,
        "access_files": facets.access_files,
        "access_record": facets.access_record,
        "access_status": facets.access_status,
        "metadata_alternateIdentifiers_alternateIdentifier": (
            facets.metadata_alternateIdentifiers_alternateIdentifier
        ),
        "metadata_alternateIdentifiers_alternateIdentifierType": (
            facets.metadata_alternateIdentifiers_alternateIdentifierType
        ),
        "metadata_container_firstPage": facets.metadata_container_firstPage,
        "metadata_container_title": facets.metadata_container_title,
        "metadata_container_type": facets.metadata_container_type,
        "metadata_contributors_affiliation_affiliationIdentifier": (
            facets.metadata_contributors_affiliation_affiliationIdentifier
        ),
        "metadata_contributors_affiliation_affiliationIdentifierScheme": (
            facets.metadata_contributors_affiliation_affiliationIdentifierScheme
        ),
        "metadata_contributors_affiliation_name": (
            facets.metadata_contributors_affiliation_name
        ),
        "metadata_contributors_affiliation_schemeURI": (
            facets.metadata_contributors_affiliation_schemeURI
        ),
        "metadata_contributors_contributorType": (
            facets.metadata_contributors_contributorType
        ),
        "metadata_contributors_familyName": facets.metadata_contributors_familyName,
        "metadata_contributors_givenName": facets.metadata_contributors_givenName,
        "metadata_contributors_lang": facets.metadata_contributors_lang,
        "metadata_contributors_name": facets.metadata_contributors_name,
        "metadata_contributors_nameIdentifiers_nameIdentifier": (
            facets.metadata_contributors_nameIdentifiers_nameIdentifier
        ),
        "metadata_contributors_nameIdentifiers_nameIdentifierScheme": (
            facets.metadata_contributors_nameIdentifiers_nameIdentifierScheme
        ),
        "metadata_contributors_nameIdentifiers_schemeURI": (
            facets.metadata_contributors_nameIdentifiers_schemeURI
        ),
        "metadata_contributors_nameType": facets.metadata_contributors_nameType,
        "metadata_creators_affiliation_affiliationIdentifier": (
            facets.metadata_creators_affiliation_affiliationIdentifier
        ),
        "metadata_creators_affiliation_affiliationIdentifierScheme": (
            facets.metadata_creators_affiliation_affiliationIdentifierScheme
        ),
        "metadata_creators_affiliation_name": facets.metadata_creators_affiliation_name,
        "metadata_creators_affiliation_schemeURI": (
            facets.metadata_creators_affiliation_schemeURI
        ),
        "metadata_creators_familyName": facets.metadata_creators_familyName,
        "metadata_creators_givenName": facets.metadata_creators_givenName,
        "metadata_creators_lang": facets.metadata_creators_lang,
        "metadata_creators_name": facets.metadata_creators_name,
        "metadata_creators_nameIdentifiers_nameIdentifier": (
            facets.metadata_creators_nameIdentifiers_nameIdentifier
        ),
        "metadata_creators_nameIdentifiers_nameIdentifierScheme": (
            facets.metadata_creators_nameIdentifiers_nameIdentifierScheme
        ),
        "metadata_creators_nameIdentifiers_schemeURI": (
            facets.metadata_creators_nameIdentifiers_schemeURI
        ),
        "metadata_creators_nameType": facets.metadata_creators_nameType,
        "metadata_dates_date": facets.metadata_dates_date,
        "metadata_dates_dateInformation": facets.metadata_dates_dateInformation,
        "metadata_dates_dateType": facets.metadata_dates_dateType,
        "metadata_descriptions_descriptionType": (
            facets.metadata_descriptions_descriptionType
        ),
        "metadata_descriptions_lang": facets.metadata_descriptions_lang,
        "metadata_doi": facets.metadata_doi,
        "metadata_formats": facets.metadata_formats,
        "metadata_fundingReferences_awardNumber": (
            facets.metadata_fundingReferences_awardNumber
        ),
        "metadata_fundingReferences_awardTitle": (
            facets.metadata_fundingReferences_awardTitle
        ),
        "metadata_fundingReferences_awardURI": (
            facets.metadata_fundingReferences_awardURI
        ),
        "metadata_fundingReferences_funderIdentifier": (
            facets.metadata_fundingReferences_funderIdentifier
        ),
        "metadata_fundingReferences_funderIdentifierType": (
            facets.metadata_fundingReferences_funderIdentifierType
        ),
        "metadata_fundingReferences_funderName": (
            facets.metadata_fundingReferences_funderName
        ),
        "metadata_geoLocations_geoLocationBox_eastBoundLongitude": (
            facets.metadata_geoLocations_geoLocationBox_eastBoundLongitude
        ),
        "metadata_geoLocations_geoLocationBox_northBoundLatitude": (
            facets.metadata_geoLocations_geoLocationBox_northBoundLatitude
        ),
        "metadata_geoLocations_geoLocationBox_southBoundLatitude": (
            facets.metadata_geoLocations_geoLocationBox_southBoundLatitude
        ),
        "metadata_geoLocations_geoLocationBox_westBoundLongitude": (
            facets.metadata_geoLocations_geoLocationBox_westBoundLongitude
        ),
        "metadata_geoLocations_geoLocationPlace": (
            facets.metadata_geoLocations_geoLocationPlace
        ),
        "metadata_geoLocations_geoLocationPoint_pointLatitude": (
            facets.metadata_geoLocations_geoLocationPoint_pointLatitude
        ),
        "metadata_geoLocations_geoLocationPoint_pointLongitude": (
            facets.metadata_geoLocations_geoLocationPoint_pointLongitude
        ),
        "metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLatitude": (
            facets.metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLatitude
        ),
        "metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLongitude": (
            facets.metadata_geoLocations_geoLocationPolygons_inPolygonPoint_pointLongitude
        ),
        "metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLatitude": (
            facets.metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLatitude
        ),
        "metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLongitude": (
            facets.metadata_geoLocations_geoLocationPolygons_polygonPoints_pointLongitude
        ),
        "metadata_language": facets.metadata_language,
        "metadata_publicationYear": facets.metadata_publicationYear,
        "metadata_publisher_lang": facets.metadata_publisher_lang,
        "metadata_publisher_name": facets.metadata_publisher_name,
        "metadata_publisher_publisherIdentifier": (
            facets.metadata_publisher_publisherIdentifier
        ),
        "metadata_publisher_publisherIdentifierScheme": (
            facets.metadata_publisher_publisherIdentifierScheme
        ),
        "metadata_publisher_schemeURI": facets.metadata_publisher_schemeURI,
        "metadata_relatedIdentifiers_relatedIdentifier": (
            facets.metadata_relatedIdentifiers_relatedIdentifier
        ),
        "metadata_relatedIdentifiers_relatedIdentifierType": (
            facets.metadata_relatedIdentifiers_relatedIdentifierType
        ),
        "metadata_relatedIdentifiers_relatedMetadataScheme": (
            facets.metadata_relatedIdentifiers_relatedMetadataScheme
        ),
        "metadata_relatedIdentifiers_relationType": (
            facets.metadata_relatedIdentifiers_relationType
        ),
        "metadata_relatedIdentifiers_resourceTypeGeneral": (
            facets.metadata_relatedIdentifiers_resourceTypeGeneral
        ),
        "metadata_relatedIdentifiers_schemeType": (
            facets.metadata_relatedIdentifiers_schemeType
        ),
        "metadata_relatedIdentifiers_schemeURI": (
            facets.metadata_relatedIdentifiers_schemeURI
        ),
        "metadata_relatedItems_contributors_affiliation_affiliationIdentifier": (
            facets.metadata_relatedItems_contributors_affiliation_affiliationIdentifier
        ),
        "metadata_relatedItems_contributors_affiliation_affiliationIdentifierScheme": (
            facets.metadata_relatedItems_contributors_affiliation_affiliationIdentifierScheme
        ),
        "metadata_relatedItems_contributors_affiliation_name": (
            facets.metadata_relatedItems_contributors_affiliation_name
        ),
        "metadata_relatedItems_contributors_affiliation_schemeURI": (
            facets.metadata_relatedItems_contributors_affiliation_schemeURI
        ),
        "metadata_relatedItems_contributors_contributorType": (
            facets.metadata_relatedItems_contributors_contributorType
        ),
        "metadata_relatedItems_contributors_familyName": (
            facets.metadata_relatedItems_contributors_familyName
        ),
        "metadata_relatedItems_contributors_givenName": (
            facets.metadata_relatedItems_contributors_givenName
        ),
        "metadata_relatedItems_contributors_lang": (
            facets.metadata_relatedItems_contributors_lang
        ),
        "metadata_relatedItems_contributors_name": (
            facets.metadata_relatedItems_contributors_name
        ),
        "metadata_relatedItems_contributors_nameIdentifiers_nameIdentifier": (
            facets.metadata_relatedItems_contributors_nameIdentifiers_nameIdentifier
        ),
        "metadata_relatedItems_contributors_nameIdentifiers_nameIdentifierScheme": (
            facets.metadata_relatedItems_contributors_nameIdentifiers_nameIdentifierScheme
        ),
        "metadata_relatedItems_contributors_nameIdentifiers_schemeURI": (
            facets.metadata_relatedItems_contributors_nameIdentifiers_schemeURI
        ),
        "metadata_relatedItems_contributors_nameType": (
            facets.metadata_relatedItems_contributors_nameType
        ),
        "metadata_relatedItems_creators_affiliation_affiliationIdentifier": (
            facets.metadata_relatedItems_creators_affiliation_affiliationIdentifier
        ),
        "metadata_relatedItems_creators_affiliation_affiliationIdentifierScheme": (
            facets.metadata_relatedItems_creators_affiliation_affiliationIdentifierScheme
        ),
        "metadata_relatedItems_creators_affiliation_name": (
            facets.metadata_relatedItems_creators_affiliation_name
        ),
        "metadata_relatedItems_creators_affiliation_schemeURI": (
            facets.metadata_relatedItems_creators_affiliation_schemeURI
        ),
        "metadata_relatedItems_creators_familyName": (
            facets.metadata_relatedItems_creators_familyName
        ),
        "metadata_relatedItems_creators_givenName": (
            facets.metadata_relatedItems_creators_givenName
        ),
        "metadata_relatedItems_creators_lang": (
            facets.metadata_relatedItems_creators_lang
        ),
        "metadata_relatedItems_creators_name": (
            facets.metadata_relatedItems_creators_name
        ),
        "metadata_relatedItems_creators_nameIdentifiers_nameIdentifier": (
            facets.metadata_relatedItems_creators_nameIdentifiers_nameIdentifier
        ),
        "metadata_relatedItems_creators_nameIdentifiers_nameIdentifierScheme": (
            facets.metadata_relatedItems_creators_nameIdentifiers_nameIdentifierScheme
        ),
        "metadata_relatedItems_creators_nameIdentifiers_schemeURI": (
            facets.metadata_relatedItems_creators_nameIdentifiers_schemeURI
        ),
        "metadata_relatedItems_creators_nameType": (
            facets.metadata_relatedItems_creators_nameType
        ),
        "metadata_relatedItems_edition": facets.metadata_relatedItems_edition,
        "metadata_relatedItems_firstPage": facets.metadata_relatedItems_firstPage,
        "metadata_relatedItems_issue": facets.metadata_relatedItems_issue,
        "metadata_relatedItems_lastPage": facets.metadata_relatedItems_lastPage,
        "metadata_relatedItems_number": facets.metadata_relatedItems_number,
        "metadata_relatedItems_numberType": facets.metadata_relatedItems_numberType,
        "metadata_relatedItems_publicationYear": (
            facets.metadata_relatedItems_publicationYear
        ),
        "metadata_relatedItems_publisher": facets.metadata_relatedItems_publisher,
        "metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifier": (
            facets.metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifier
        ),
        "metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifierType": (
            facets.metadata_relatedItems_relatedItemIdentifier_relatedItemIdentifierType
        ),
        "metadata_relatedItems_relatedItemType": (
            facets.metadata_relatedItems_relatedItemType
        ),
        "metadata_relatedItems_relatedMetadataScheme": (
            facets.metadata_relatedItems_relatedMetadataScheme
        ),
        "metadata_relatedItems_relationType": facets.metadata_relatedItems_relationType,
        "metadata_relatedItems_resourceTypeGeneral": (
            facets.metadata_relatedItems_resourceTypeGeneral
        ),
        "metadata_relatedItems_schemeType": facets.metadata_relatedItems_schemeType,
        "metadata_relatedItems_schemeURI": facets.metadata_relatedItems_schemeURI,
        "metadata_relatedItems_titles_lang": facets.metadata_relatedItems_titles_lang,
        "metadata_relatedItems_titles_titleType": (
            facets.metadata_relatedItems_titles_titleType
        ),
        "metadata_relatedItems_volume": facets.metadata_relatedItems_volume,
        "metadata_resourceType_resourceType": facets.metadata_resourceType_resourceType,
        "metadata_resourceType_resourceTypeGeneral": (
            facets.metadata_resourceType_resourceTypeGeneral
        ),
        "metadata_rightsList_lang": facets.metadata_rightsList_lang,
        "metadata_rightsList_rights": facets.metadata_rightsList_rights,
        "metadata_rightsList_rightsIdentifier": (
            facets.metadata_rightsList_rightsIdentifier
        ),
        "metadata_rightsList_rightsIdentifierScheme": (
            facets.metadata_rightsList_rightsIdentifierScheme
        ),
        "metadata_rightsList_rightsURI": facets.metadata_rightsList_rightsURI,
        "metadata_rightsList_schemeURI": facets.metadata_rightsList_schemeURI,
        "metadata_schemaVersion": facets.metadata_schemaVersion,
        "metadata_sizes": facets.metadata_sizes,
        "metadata_subjects_classificationCode": (
            facets.metadata_subjects_classificationCode
        ),
        "metadata_subjects_lang": facets.metadata_subjects_lang,
        "metadata_subjects_schemeURI": facets.metadata_subjects_schemeURI,
        "metadata_subjects_subject": facets.metadata_subjects_subject,
        "metadata_subjects_subjectScheme": facets.metadata_subjects_subjectScheme,
        "metadata_subjects_valueURI": facets.metadata_subjects_valueURI,
        "metadata_titles_lang": facets.metadata_titles_lang,
        "metadata_titles_titleType": facets.metadata_titles_titleType,
        "metadata_url": facets.metadata_url,
        "metadata_version": facets.metadata_version,
        **getattr(I18nRDMSearchOptions, "facets", {}),
        "record_status": facets.record_status,
        "has_draft": facets.has_draft,
    }



from invenio_rdm_records.services.config import RDMRecordServiceConfig
from invenio_records_resources.services import (
    ConditionalLink,
    LinksTemplate,
    RecordLink,
    pagination_links,
)
from oarepo_runtime.services.components import (
    CustomFieldsComponent,
    process_service_configs,
)
from oarepo_runtime.services.config import (
    has_draft,
    has_permission,
    has_published_record,
    is_published_record,
)
from oarepo_runtime.services.config.service import PermissionsPresetsConfigMixin
from oarepo_runtime.services.records import pagination_links_html

from nr_metadata.datacite.records.api import DataciteRecord
from nr_metadata.datacite.services.records.permissions import DatacitePermissionPolicy
from nr_metadata.datacite.services.records.results import (
    DataciteRecordItem,
    DataciteRecordList,
)
from nr_metadata.datacite.services.records.schema import DataCiteRecordSchema
from nr_metadata.datacite.services.records.search import DataciteSearchOptions


class DataciteServiceConfig(PermissionsPresetsConfigMixin, RDMRecordServiceConfig):
    """DataciteRecord service config."""

    result_item_cls = DataciteRecordItem

    result_list_cls = DataciteRecordList

    PERMISSIONS_PRESETS = ["everyone"]

    url_prefix = "/nr-metadata-datacite/"

    base_permission_policy_cls = DatacitePermissionPolicy

    schema = DataCiteRecordSchema

    search = DataciteSearchOptions

    record_cls = DataciteRecord

    service_id = "datacite"

    search_item_links_template = LinksTemplate

    @property
    def components(self):
        return process_service_configs(self, CustomFieldsComponent)

    model = "nr_metadata.datacite"

    @property
    def links_item(self):
        return {
            "draft": RecordLink(
                "{+api}/nr-metadata-datacite/{id}/draft",
                when=has_draft() & has_permission("read_draft"),
            ),
            "edit_html": RecordLink(
                "{+ui}/nr-metadata-datacite/{id}/edit",
                when=has_draft() & has_permission("update"),
            ),
            "latest": RecordLink(
                "{+api}/nr-metadata-datacite/{id}/versions/latest",
                when=has_permission("read"),
            ),
            "latest_html": RecordLink(
                "{+ui}/nr-metadata-datacite/{id}/latest", when=has_permission("read")
            ),
            "publish": RecordLink(
                "{+api}/nr-metadata-datacite/{id}/draft/actions/publish",
                when=has_permission("publish"),
            ),
            "record": RecordLink(
                "{+api}/nr-metadata-datacite/{id}",
                when=has_published_record() & has_permission("read"),
            ),
            "self": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+api}/nr-metadata-datacite/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+api}/nr-metadata-datacite/{id}/draft",
                    when=has_permission("read_draft"),
                ),
            ),
            "self_html": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+ui}/nr-metadata-datacite/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+ui}/nr-metadata-datacite/{id}/preview",
                    when=has_permission("read_draft"),
                ),
            ),
            "versions": RecordLink(
                "{+api}/nr-metadata-datacite/{id}/versions",
                when=has_permission("search_versions"),
            ),
        }

    @property
    def links_search_item(self):
        return {
            "self": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+api}/nr-metadata-datacite/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+api}/nr-metadata-datacite/{id}/draft",
                    when=has_permission("read_draft"),
                ),
            ),
            "self_html": ConditionalLink(
                cond=is_published_record(),
                if_=RecordLink(
                    "{+ui}/nr-metadata-datacite/{id}", when=has_permission("read")
                ),
                else_=RecordLink(
                    "{+ui}/nr-metadata-datacite/{id}/preview",
                    when=has_permission("read_draft"),
                ),
            ),
        }

    @property
    def links_search(self):
        return {
            **pagination_links("{+api}/nr-metadata-datacite/{?args*}"),
            **pagination_links_html("{+ui}/nr-metadata-datacite/{?args*}"),
        }

    @property
    def links_search_drafts(self):
        return {
            **pagination_links("{+api}/user/nr-metadata-datacite/{?args*}"),
            **pagination_links_html("{+ui}/user/nr-metadata-datacite/{?args*}"),
        }

    @property
    def links_search_versions(self):
        return {
            **pagination_links("{+api}/nr-metadata-datacite/{id}/versions{?args*}"),
        }



import marshmallow as ma
from edtf import Interval as EDTFInterval
from invenio_drafts_resources.services.records.schema import (
    ParentSchema as InvenioParentSchema,
)
from invenio_rdm_records.services.schemas.access import AccessSchema
from invenio_rdm_records.services.schemas.metadata import CreatorSchema
from invenio_rdm_records.services.schemas.pids import PIDSchema
from invenio_rdm_records.services.schemas.record import validate_scheme
from invenio_vocabularies.services.schema import i18n_strings
from marshmallow import fields as ma_fields
from marshmallow.fields import Dict, Nested, String
from marshmallow_utils.fields import SanitizedUnicode, TrimmedString
from marshmallow_utils.fields.nestedattr import NestedAttribute
from oarepo_runtime.services.schema.marshmallow import (
    DictOnlySchema,
    RDMBaseRecordSchema,
)
from oarepo_runtime.services.schema.rdm import FundingSchema
from oarepo_runtime.services.schema.validation import (
    CachedMultilayerEDTFValidator,
    validate_date,
    validate_identifier,
)
from oarepo_vocabularies.services.schema import HierarchySchema

from nr_metadata.common.services.records.schema_common import (
    AdditionalTitlesSchema,
    NRCommonMetadataSchema,
)
from nr_metadata.common.services.records.schema_datatypes import (
    NREventSchema,
    NRGeoLocationSchema,
    NRRelatedItemSchema,
    NRSeriesSchema,
    NRSubjectSchema,
)
from nr_metadata.schema.identifiers import (
    NRObjectIdentifierSchema,
    NRSystemIdentifierSchema,
)


class GeneratedParentSchema(InvenioParentSchema):
    """"""

    owners = ma.fields.List(ma.fields.Dict(), load_only=True)


class NRDataRecordSchema(RDMBaseRecordSchema):
    class Meta:
        unknown = ma.RAISE

    access = NestedAttribute(lambda: AccessSchema())

    metadata = ma_fields.Nested(lambda: NRDataMetadataSchema())

    pids = Dict(
        keys=SanitizedUnicode(validate=validate_scheme),
        values=Nested(PIDSchema),
    )
    parent = ma.fields.Nested(GeneratedParentSchema)


class NRDataMetadataSchema(NRCommonMetadataSchema):
    class Meta:
        unknown = ma.RAISE

    additionalTitles = ma_fields.List(
        ma_fields.Nested(lambda: AdditionalTitlesSchema())
    )

    contributors = ma_fields.List(ma_fields.Nested(lambda: CreatorSchema()))

    creators = ma_fields.List(
        ma_fields.Nested(lambda: CreatorSchema()),
        required=True,
        validate=[ma.validate.Length(min=1)],
    )

    dateCollected = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFInterval,))]
    )

    dateCreated = TrimmedString(
        validate=[CachedMultilayerEDTFValidator(types=(EDTFInterval,))]
    )

    dateValidTo = ma_fields.String(validate=[validate_date("%Y-%m-%d")])

    dateWithdrawn = ma_fields.Nested(lambda: DateWithdrawnSchema())

    events = ma_fields.List(ma_fields.Nested(lambda: NREventSchema()))

    funders = ma_fields.List(ma_fields.Nested(lambda: FundingSchema()))

    geoLocations = ma_fields.List(ma_fields.Nested(lambda: NRGeoLocationSchema()))

    objectIdentifiers = ma_fields.List(
        ma_fields.Nested(
            lambda: NRObjectIdentifierSchema(),
            validate=[lambda value: validate_identifier(value)],
        )
    )

    publishers = ma_fields.List(
        ma_fields.Nested(lambda: PublishersItemSchema()),
        validate=[ma.validate.Length(min=1)],
    )

    relatedItems = ma_fields.List(ma_fields.Nested(lambda: NRRelatedItemSchema()))

    series = ma_fields.List(ma_fields.Nested(lambda: NRSeriesSchema()))

    subjects = ma_fields.List(ma_fields.Nested(lambda: NRSubjectSchema()))

    systemIdentifiers = ma_fields.List(
        ma_fields.Nested(lambda: NRSystemIdentifierSchema())
    )


class DateWithdrawnSchema(DictOnlySchema):
    class Meta:
        unknown = ma.RAISE

    dateInformation = ma_fields.String()

    type = ma_fields.String(validate=[validate_date("%Y-%m-%d")])


class PublishersItemSchema(DictOnlySchema):
    class Meta:
        unknown = ma.INCLUDE

    _id = String(data_key="id", attribute="id")

    _version = String(data_key="@v", attribute="@v")

    hierarchy = ma_fields.Nested(lambda: HierarchySchema())

    ror = ma_fields.String()

    title = i18n_strings



from invenio_db import db
from invenio_drafts_resources.records import ParentRecordMixin
from invenio_files_rest.models import Bucket
from invenio_rdm_records.records.systemfields.deletion_status import (
    RecordDeletionStatusEnum,
)
from invenio_records.models import RecordMetadataBase
from sqlalchemy_utils.types import ChoiceType, UUIDType


class CommonParentMetadata(db.Model, RecordMetadataBase):

    __tablename__ = "nr_metadata.common_parent_record_metadata"


class CommonMetadata(db.Model, RecordMetadataBase, ParentRecordMixin):
    """Model for CommonRecord metadata."""

    __tablename__ = "common_metadata"

    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}

    __parent_record_model__ = CommonParentMetadata

    deletion_status = db.Column(
        ChoiceType(RecordDeletionStatusEnum, impl=db.String(1)),
        nullable=False,
        default=RecordDeletionStatusEnum.PUBLISHED.value,
    )

    media_bucket_id = db.Column(UUIDType, db.ForeignKey(Bucket.id), index=True)
    media_bucket = db.relationship(Bucket, foreign_keys=[media_bucket_id])



from oarepo_runtime.records.dumpers import SearchDumper
from oarepo_runtime.records.systemfields.mapping import SystemFieldDumperExt

from nr_metadata.common.records.dumpers.edtf import CommonEDTFIntervalDumperExt
from nr_metadata.common.records.dumpers.multilingual import MultilingualSearchDumperExt


class CommonDumper(SearchDumper):
    """CommonRecord opensearch dumper."""

    extensions = [
        SystemFieldDumperExt(),
        CommonEDTFIntervalDumperExt(),
        MultilingualSearchDumperExt(),
    ]
